@@ -1,8 +1,8 @@
 fileToGeoJSON <-
 function(data, name, dest, overwrite) {
-	if(!file.exists(data)) stop("data file not found")
-	if(file.exists(paste0(file.path(dest, name), ".geojson")) && !overwrite) stop("abort - file already exists\n")
-	stopifnot(require(httr))
+	if(!file.exists(data)) stop("Data file not found")
+	if(file.exists(paste0(file.path(dest, name), ".geojson")) && !overwrite) stop("Abort - file already exists")
+	stopifnot(require(httr, quietly=TRUE))
 	
 	# taken from rgbif package: cran.r-project.org/package=rgbif‎
 	# package import impractical, since rgbif imports several other packages
@@ -15,4 +15,4 @@ function(data, name, dest, overwrite) {
     # end rgbif code
 
 	return(paste0(file.path(dest, name), ".geojson"))
-}
\ No newline at end of file
+}
